use actix_web::{web, HttpResponse, Responder};
use rusqlite::{params, Connection};
use serde::{Deserialize, Serialize};
use std::sync::{Arc, Mutex};
use uuid::Uuid;

use crate::auth::Auth;

// Structs for requests and responses
#[derive(Deserialize)]
pub struct LoginRequest {
    pub username: String,
    pub password: String,
}

#[derive(Serialize)]
pub struct LoginResponse {
    pub token: String,
}

// Public: Login handler
pub async fn login(
    db: web::Data<Arc<Mutex<Connection>>>,
    login_request: web::Json<LoginRequest>,
) -> impl Responder {
    let conn = db.lock().unwrap();
    let user_id =
        match crate::db::authenticate_user(&conn, &login_request.username, &login_request.password)
        {
            Ok(Some(user_id)) => user_id,
            Ok(None) => return HttpResponse::Unauthorized().body("Invalid username or password"),
            Err(_) => return HttpResponse::InternalServerError().body("Database error"),
        };

    let token = Uuid::new_v4().to_string();
    if let Err(_) = crate::db::generate_token_for_user(&conn, &user_id, &token) {
        return HttpResponse::InternalServerError().body("Failed to generate token");
    }

    HttpResponse::Ok().json(LoginResponse { token })
}

// Public: Submit a form
pub async fn submit_form(
    db: web::Data<Arc<Mutex<Connection>>>,
    path: web::Path<String>,
    submission: web::Form<serde_json::Value>,
) -> impl Responder {
    let conn = db.lock().unwrap();
    let submission_id = Uuid::new_v4().to_string();
    let form_id = path.into_inner();
    let submission_json = serde_json::to_string(&submission.into_inner()).unwrap();

    match conn.execute(
        "INSERT INTO submissions (id, form_id, data) VALUES (?1, ?2, ?3)",
        params![submission_id, form_id, submission_json],
    ) {
        Ok(_) => HttpResponse::Ok().json(submission_id),
        Err(e) => HttpResponse::InternalServerError().body(format!("Error: {}", e)),
    }
}

// Protected: Create a new form
pub async fn create_form(
    db: web::Data<Arc<Mutex<Connection>>>,
    auth: Auth,
    form: web::Json<crate::models::Form>,
) -> impl Responder {
    println!("Authenticated user: {}", auth.user_id);
    let conn = db.lock().unwrap();
    let form_id = Uuid::new_v4().to_string();
    let form_json = serde_json::to_string(&form.fields).unwrap();

    match conn.execute(
        "INSERT INTO forms (id, name, fields) VALUES (?1, ?2, ?3)",
        params![form_id, form.name, form_json],
    ) {
        Ok(_) => HttpResponse::Ok().json(form_id),
        Err(e) => HttpResponse::InternalServerError().body(format!("Error: {}", e)),
    }
}

// Protected: Get all forms
pub async fn get_forms(db: web::Data<Arc<Mutex<Connection>>>, auth: Auth) -> impl Responder {
    println!("Authenticated user: {}", auth.user_id);
    let conn = db.lock().unwrap();

    let mut stmt = match conn.prepare("SELECT id, name, fields FROM forms") {
        Ok(stmt) => stmt,
        Err(e) => return HttpResponse::InternalServerError().body(format!("Error: {}", e)),
    };

    let forms_iter = stmt
        .query_map([], |row| {
            let id: Option<String> = row.get(0)?;
            let name: String = row.get(1)?;
            let fields: String = row.get(2)?;
            let fields = serde_json::from_str(&fields).unwrap();
            Ok(crate::models::Form { id, name, fields })
        })
        .unwrap();

    let forms: Vec<crate::models::Form> = forms_iter.filter_map(|f| f.ok()).collect();
    HttpResponse::Ok().json(forms)
}

// Protected: Get submissions for a form
pub async fn get_submissions(
    db: web::Data<Arc<Mutex<Connection>>>,
    auth: Auth,
    path: web::Path<String>,
) -> impl Responder {
    println!("Authenticated user: {}", auth.user_id);
    let conn = db.lock().unwrap();
    let form_id = path.into_inner();

    let mut stmt =
        match conn.prepare("SELECT id, form_id, data FROM submissions WHERE form_id = ?1") {
            Ok(stmt) => stmt,
            Err(e) => return HttpResponse::InternalServerError().body(format!("Error: {}", e)),
        };

    let submissions_iter = stmt
        .query_map([form_id], |row| {
            let id: String = row.get(0)?;
            let form_id: String = row.get(1)?;
            let data: String = row.get(2)?;
            let data = serde_json::from_str(&data).unwrap();
            Ok(crate::models::Submission { id, form_id, data })
        })
        .unwrap();

    let submissions: Vec<crate::models::Submission> =
        submissions_iter.filter_map(|s| s.ok()).collect();
    HttpResponse::Ok().json(submissions)
}
